use derive_more::{From, Deref, with_trait::Display as Display};
use serde::{Serialize, Serializer, Deserialize, Deserializer};
use std::{
    fmt::{Debug, Formatter, Result as FmtResult},
    time::Duration,
};

#[derive(
    Clone,
    PartialEq,
    Eq,
    PartialOrd,
    Ord,
    From,
    Deref,
)]
pub struct FmtRelativeTime(pub Duration);

impl Display for FmtRelativeTime {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        let formatter = timeago::Formatter::new();

        write!(f, "{}", formatter.convert(self.0))
    }
}

impl Debug for FmtRelativeTime {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        Display::fmt(self, f)
    }
}

impl Serialize for FmtRelativeTime {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        serializer.serialize_u64(self.0.as_secs())
    }
}

impl<'de> Deserialize<'de> for FmtRelativeTime {
    fn deserialize<D>(deserializer: D) -> Result<FmtRelativeTime, D::Error>
    where
        D: Deserializer<'de>,
    {
        let secs = u64::deserialize(deserializer)?;
        Ok(FmtRelativeTime(Duration::from_secs(secs)))
    }
}