use derive_more::{Display, From, Deref, with_trait::Display as TDisplay};
use serde::{Serialize, Serializer, Deserialize, Deserializer};
use std::{
    fmt::{Debug, Formatter, Result as FmtResult},
    str::FromStr,
};
use versions::Versioning;

#[derive(Clone, PartialEq, Eq, PartialOrd, Ord, From, Deref, Display)]
#[display("{_0}")]
pub struct FmtVersion(pub Versioning);

impl Debug for FmtVersion {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        TDisplay::fmt(self, f)
    }
}

impl Serialize for FmtVersion {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        serializer.serialize_str(&self.to_string())
    }
}

impl<'de> Deserialize<'de> for FmtVersion {
    fn deserialize<D>(deserializer: D) -> Result<FmtVersion, D::Error>
    where
        D: Deserializer<'de>,
    {
        let s = String::deserialize(deserializer)?;
        Versioning::from_str(&s)
            .map(FmtVersion)
            .map_err(serde::de::Error::custom)
    }
}